@@ -179,6 +179,24 @@ function isValidVector(obj) {
     return (isVector(obj) && obj.isValid() && isNumber(obj.x) && isNumber(obj.y));
 }
 
+/**
+ * Checks if the given object is a color.
+ *
+ * \return true if the given object is an RColor object.
+ */
+function isColor(obj) {
+    return (typeof(obj)==="object" && isOfType(obj, RColor));
+}
+
+/**
+ * Checks if the given object is a valid color.
+ *
+ * \return true if the given object is a valid RColor object.
+ */
+function isValidColor(obj) {
+    return (isColor(obj) && obj.isValid());
+}
+
 /**
  * Checks if the given object is a valid boolean value.
  *
@@ -552,6 +570,15 @@ function isSplineEntity(obj) {
     return isOfType(obj, RSplineEntity) || isOfType(obj, RSplineEntityPointer);
 }
 
+/**
+ * Checks if the given object is a spline entity with fit points.
+ *
+ * \return true if the given object is a spline entity (RSplineEntity) with fit points.
+ */
+function isSplineEntityWithFitPoints(obj) {
+    return isSplineEntity(obj) && obj.hasFitPoints();
+}
+
 function isClosedSplineEntity(obj, tolerance) {
     if (isNull(tolerance)) {
         return isSplineEntity(obj) && obj.isGeometricallyClosed();
@@ -818,6 +845,100 @@ function isTriangleShape(obj) {
     return isOfType(obj, RTriangle) || isOfType(obj, RTrianglePointer);
 }
 
+/**
+ * Converts the given entity type enum to the class name.
+ *
+ * \return Class name (e.g. "RLineEntity" for RS.EntityLine
+ *
+ * \param type Entity type (RS::EntityType)
+ */
+function entityTypeToClass(type) {
+    switch (type) {
+    case RS.ObjectAll:
+        return "RObject";
+    case RS.ObjectUnknown:
+        return "RObject";
+    case RS.ObjectBlock:
+        return "RBlock";
+    case RS.ObjectLayer:
+        return "RLayer";
+    case RS.ObjectLayout:
+        return "RLayout";
+    case RS.ObjectLinetype:
+        return "RLinetype";
+    case RS.ObjectView:
+        return "RView";
+
+    case RS.EntityAll:
+        return "REntity";
+    case RS.EntityBlockRef:
+        return "RBlockReferenceEntity";
+    case RS.EntityArc:
+        return "RArcEntity";
+    case RS.EntityAttribute:
+        return "RAttributeEntity";
+    case RS.EntityAttributeDefinition:
+        return "RAttributeDefinitionEntity";
+    case RS.EntityCircle:
+        return "RCircleEntity";
+    case RS.EntityDimension:
+        return "RDimensionEntity";
+    case RS.EntityDimAligned:
+        return "RDimAlignedEntity";
+    case RS.EntityDimAngular:
+        return "RDimAngularEntity";
+    case RS.EntityDimAngular2L:
+        return "RDimAngular2LEntity";
+    case RS.EntityDimAngular3P:
+        return "RDimAngular3PEntity";
+    case RS.EntityDimArcLength:
+        return "RDimArcLengthEntity";
+    case RS.EntityDimDiametric:
+        return "RDimDiametricEntity";
+    case RS.EntityDimOrdinate:
+        return "RDimOrdinateEntity";
+    case RS.EntityDimRotated:
+        return "RDimRotatedEntity";
+    case RS.EntityDimRadial:
+        return "RDimRadialEntity";
+    case RS.EntityEllipse:
+        return "REllipseEntity";
+    case RS.EntityHatch:
+        return "RHatchEntity";
+    case RS.EntityImage:
+        return "RImageEntity";
+    case RS.EntityLeader:
+        return "RLeaderEntity";
+    case RS.EntityTolerance:
+        return "RToleranceEntity";
+    case RS.EntityLine:
+        return "RLineEntity";
+    case RS.EntityXLine:
+        return "RXLineEntity";
+    case RS.EntityRay:
+        return "RRayEntity";
+    case RS.EntityPoint:
+        return "RPointEntity";
+    case RS.EntityPolyline:
+        return "RPolylineEntity";
+    case RS.EntitySolid:
+        return "RSolidEntity";
+    case RS.EntityTrace:
+        return "RTraceEntity";
+    case RS.EntityFace:
+        return "RFaceEntity";
+    case RS.EntitySpline:
+        return "RSplineEntity";
+    case RS.EntityTextBased:
+        return "RTextBasedEntity";
+    case RS.EntityText:
+        return "RTextEntity";
+    case RS.EntityViewport:
+        return "RViewportEntity";
+    default:
+        return undefined;
+    }
+}
 
 /**
  * Converts the given entity type enum to a human readable,
@@ -925,6 +1046,102 @@ function entityTypeToString(type, plural) {
     }
 }
 
+/**
+ * Converts the given entity type name to the entity type enum.
+ *
+ * \return entity type enum.
+ *
+ * \param typeName Entity type name ("Line", "Arc", ...)
+ */
+function getEntityType(typeName) {
+    switch (typeName) {
+    case "Block":
+        return RS.ObjectBlock;
+    case "Layer":
+        return RS.ObjectLayer;
+    case "Layout":
+        return RS.ObjectLayout;
+    case "Linetype":
+        return RS.ObjectLinetype;
+    case "View":
+        return RS.ObjectView;
+
+    case "Entity":
+        return RS.EntityAll;
+    case "3dFace":
+        return RS.Entity3dFace;
+    case "Block Reference":
+        return RS.EntityBlockRef;
+    case "Block Reference and Attributes":
+        return RS.EntityBlockRefAttr;
+    case "Arc":
+        return RS.EntityArc;
+    case "Attribute":
+        return RS.EntityAttribute;
+    case "Attribute Definition":
+        return RS.EntityAttributeDefinition;
+    case "Circle":
+        return RS.EntityCircle;
+    case "Dimension":
+        return RS.EntityDimension;
+    case "Aligned Dimension":
+        return RS.EntityDimAligned;
+    case "Angular Dimension":
+        return RS.EntityDimAngular;
+    case "Angular Dimension (2 Line)":
+        return RS.EntityDimAngular2L;
+    case "Angular Dimension (3 Point)":
+        return RS.EntityDimAngular3P;
+    case "Arc Dimension":
+        return RS.EntityDimArcLength;
+    case "Diametric Dimension":
+        return RS.EntityDimDiametric;
+    case "Ordinate Dimension":
+        return RS.EntityDimOrdinate;
+    case "Rotated Dimension":
+        return RS.EntityDimRotated;
+    case "Radial Dimension":
+        return RS.EntityDimRadial;
+    case "Ellipse":
+        return RS.EntityEllipse;
+    case "Hatch":
+        return RS.EntityHatch;
+    case "Image":
+        return RS.EntityImage;
+    case "Leader":
+        return RS.EntityLeader;
+    case "Tolerance":
+        return RS.EntityTolerance;
+    case "Line":
+        return RS.EntityLine;
+    case "Infinite Line":
+    case "XLine":
+        return RS.EntityXLine;
+    case "Ray":
+        return RS.EntityRay;
+    case "Point":
+        return RS.EntityPoint;
+    case "Polyline":
+        return RS.EntityPolyline;
+    case "Solid":
+        return RS.EntitySolid;
+    case "Trace":
+        return RS.EntityTrace;
+    case "Face":
+        return RS.EntityFace;
+    case "Spline":
+        return RS.EntitySpline;
+    case "Text based":
+        return RS.EntityTextBased;
+    case "Text":
+        return RS.EntityText;
+    case "Viewport":
+        return RS.EntityViewport;
+    default:
+        return RS.ObjectUnknown;
+    }
+}
+
 function translateFilterStrings(filterStrings) {
     for (var i=0; i<filterStrings.length; i++) {
         filterStrings[i] = filterStrings[i].replace("All CAD Files", qsTr("All CAD Files"));
@@ -1048,20 +1265,6 @@ function getWidgets(widget, ca, allowDuplicates) {
     return ca;
 }
 
-/**
- * Sleeps for the given milliseconds.
- * \param msec the time to sleep
- */
-function sleep(msec) {
-    var date = new Date();
-    var curDate = null;
-    var c = 0;
-    do {
-        curDate = new Date();
-        ++c;
-    } while (curDate - date < msec);
-}
-
 /**
  * Converts radiant to degrees.
  * \param val the value in radiant
@@ -2038,7 +2241,7 @@ function mergeProperties(obj1,obj2) {
  * (i.e. contains invalid characters).
  */
 function fixSymbolTableName(name) {
-    // trim white space at begining and end to avoid invalid block name:
+    // trim white space at beginning and end to avoid invalid block name:
     name = name.trim();
 
     // max length:
@@ -2160,9 +2363,9 @@ function initUserShortcuts() {
 
         if (match) qDebug("scStringList", scStringList);
 
-        // explicitely no shortcuts:
+        // explicitly no shortcuts:
         if (isNull(scStringList)) {
-            if (match) qDebug("explicitely no shortcuts");
+            if (match) qDebug("explicitly no shortcuts");
             action.setShortcuts([]);
             continue;
         }
@@ -2190,7 +2393,7 @@ function initUserShortcuts() {
         }
         var cmStringList = RSettings.getValue("Commands/" + key);
 
-        // explicitely no commands:
+        // explicitly no commands:
         if (isNull(cmStringList)) {
             action.setCommands([]);
             continue;
@@ -2390,14 +2593,55 @@ function neutralPath(path) {
     return path;
 }
 
+/**
+ * \return Mapped icon path for the given icon for the current theme or
+ * mapped icon path for a dark theme (-inverse) or the same path or undefined if no such
+ * icon can be found.
+ */
 function autoIconPath(path) {
-    if (RSettings.hasDarkGuiBackground()) {
-        var pathDark = path.replace(/\.svg$/, "-inverse.svg");
-        if (new QFileInfo(pathDark).exists()) {
-            return pathDark;
+    // set theme specific icon:
+    var themePath = RSettings.getThemePath();
+    var themeIconFile = undefined;
+    if (themePath.length>0) {
+        var fi = new QFileInfo(path);
+        var iconFileName = fi.fileName();
+        themeIconFile = themePath + "/icons/" + iconFileName;
+        if (!new QFileInfo(themeIconFile).exists()) {
+            // no SVG found, look up PNG:
+            var iconBaseName = fi.baseName();
+            themeIconFile = themePath + "/icons/" + iconBaseName + ".png";
+            if (!new QFileInfo(themeIconFile).exists()) {
+                // no PNG found, use default icon:
+                themeIconFile = undefined;
+            }
         }
     }
 
+    if (!isNull(themeIconFile)) {
+        // got icon from theme:
+        return themeIconFile;
+    }
+
+    // no theme icon, try dark mode icon:
+    var darkModeIconFile = undefined;
+    if (RSettings.hasDarkGuiBackground()) {
+        darkModeIconFile = path.replace(/\.svg$/, "-inverse.svg");
+        darkModeIconFile = darkModeIconFile.replace(/\.png$/, "-inverse.png");
+        if (!new QFileInfo(darkModeIconFile).exists()) {
+            darkModeIconFile = undefined;
+        }
+    }
+
+    if (!isNull(darkModeIconFile)) {
+        // got dark mode icon:
+        return darkModeIconFile;
+    }
+
+    if (!new QFileInfo(path).exists()) {
+        // given icon does not exist (for icons based on object names, e.g. in options toolbar):
+        return undefined;
+    }
+
     return path;
 }
 
@@ -2466,8 +2710,8 @@ function writeTextFile(fileName, str) {
         var textStream = new QTextStream(file);
         textStream.setCodec("UTF-8");
         textStream.writeString(str);
+        file.close();
     }
-    file.close();
 }
 
 function getKeyboardModifiers(event) {
